# Copyright 1999-2004 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2
# $Header: $

inherit eutils

DESCRIPTION="Music streaming server for SLIMP3/Squeezebox devices or software players."
HOMEPAGE="http://www.slimdevices.com"
MY_P="SlimServer_v${PV}"
SRC_URI="http://www.slimdevices.com/downloads/${MY_P}/${MY_P}.no-cpan-arch.tar.gz"

#
# NOTE:	Added ~amd64, since that's my server's arch =)
#
LICENSE="GPL-2"
SLOT="0"
KEYWORDS="~x86 ~amd64"
RESTRICT="nostrip"

IUSE="encode flac aac vorbis shorten musepack mp3 wma"
#
# NOTE:	I changed this: shorten is now a global USE flag, and there's
#		another package that uses "musepack" instead of "mpc".  Going for
#		some consistency that is the insanity of USE flags.
#
#		For profiles/use.local.desc:
#			media-sound/slimserver:musepack - Adds support for Musepack files
#			media-sound/slimserver:wma - Configures support for WMA files
#
# NOTE:	Flags "musepack" and "shorten" should be use.masked on archs which
#		don't have media-sound/musepack-tools and media-sound/shorten
#		packages yet.

DEPEND=">=dev-lang/perl-5.8.1
	>=sys-apps/sed-4"

RDEPEND=">=dev-lang/perl-5.8.1
	encode? ( >=media-sound/lame-3.96 )
	flac? ( >=media-libs/flac-1.1 )
	media-video/ffmpeg
	
	dev-perl/Audio-Wav
	dev-perl/Class-DBI
	dev-perl/Compress-Zlib
	dev-perl/DBD-SQLite
	dev-perl/DBI
	dev-perl/File-BOM
	dev-perl/File-Which
	dev-perl/HTML-Parser
	dev-perl/libwww-perl
	dev-perl/Template-Toolkit
	dev-perl/URI
	dev-perl/XML-Parser
	dev-perl/XML-Simple
	perl-core/Digest-MD5
	perl-core/MIME-Base64
	perl-core/Time-HiRes
	flac? ( dev-perl/Audio-FLAC-Header )
	musepack? ( dev-perl/Audio-Musepack )
	wma? ( dev-perl/Audio-WMA )
	vorbis? ( dev-perl/Ogg-Vorbis-Header-PurePerl )
	mp3? ( dev-perl/MP3-Info )
	dev-perl/QuickTime-Movie
	dev-perl/SQL-Abstract
	dev-perl/SQL-Abstract-Limit
	dev-perl/Tie-Cache-LRU
	dev-perl/Tie-Cache-LRU-Expires
	dev-perl/Tie-RegexpHash"
	
# NOTE:	I added the ffmpeg dependency for several reasons.  If you look
#		at the audio formats that ffmpeg supports, it looks like that may
#		actually be the most flexible way to set up decoders for the
#		SlimServer.  The exception being using FLAC cue sheets; I need to
#		examine that further.  Ffmpeg supports more formats, most notably
#		it can determine between the AAC and ALAC (Apple Lossles) codecs
#		in iTunes-created .m4a files at runtime, which is something I
#		need greatly, and I suspect others will as well.  Ffmpeg seems
#		like a smart, flexible way to do things.  I realize it's a huge
#		package, but I think the benefits are worth it.
#
#		Thoughts, anyone?
#
#		I left the USE flags so we can still use them here to determine
#		which file formats should be enabled/disabled in the various
#		slimserver config files.
#
#		The dependencies on LAME and FLAC are for using them as an
#		encoder (to stream FLAC or MP3 to the slim server), but *not*
#		to function as the decoder.  Ffmpeg is much more flexible and
#		extensible for that purpose.  It also simplifies the ebuild
#		logic =)
#

S="${WORKDIR}/${MY_P}"

#
# Remember our old configuration values, or new user-defined ones, much
# like apache does for modules in its ebuild.
#
[ -f "/etc/conf.d/slimserver" ]	&& source "/etc/conf.d/slimserver"

#
# System installation-related paths
#
# NOTE:	I changed this to be in /opt since it behaves more like a binary
#		package as far as included files.  This is a more FHS compliant
#		behavior for the package.
#
#		Still letting the user override these, but it's probably a bad idea
#		in the long run, except SLIM_LOG_FILE, which users may want to
#		change on their end.
#
# NOTE:	To be fully FHS compliant, /etc/slimserver.conf should really be
#		/etc/opt/slimserver/slimserver.conf, but that seems excessive,
#		and /etc/slimserver.conf seems to fit in with Gentoo better
#
# NOTE:	Also, though /etc/slimserver.conf feels right, it must be noted
#		that the SlimServer will modify it's configuration file WHILE
#		RUNNING, so /etc seems like a bad place for that to be.  Actually,
#		I decided to put it in /etc/slimserver/slimserver.conf so we have
#		a folder in which to remember the values for the following variables,
#		like apache does for modules.  See below.
#
#		Still don't know how to handle the fact that SlimServer will
#		modify its config file, because anywhere in /etc still seems like
#		a bad idea for that to be happening, especially if we will be
#		running the server as a non-privileged user.  Feedback welcome.
#		For now, we just make /etc/slimserver/slimserver.conf belong
#		to ${SLIM_USER}:${SLIM_GROUP}
#
[ -z "${SLIM_INSTALL_DIR}" ]	&& SLIM_INSTALL_DIR="/opt/slimserver"

#
# User-configurable variables
#
# NOTE:	According to FHS, I think it may be appropriate to locate music
#		files in /srv/music/[somewhere], since /usr/share is for "static"
#		data, and a user will likely be adding/removing music all the
#		time.  For the multitude of music folders here, /srv/music seems
#		better, even though /usr/share/music seems right for music only.
#		However, that leaves playlists and iTunes out in the cold, so
#		I'm venturing into /srv.  This is very open for debate, but my
#		needs and FHS seem to be on my side for now.  Comments are
#		appreciated.  =)
#
# NOTE:	Added a SLIM_ITUNES_DIR for those of us brave enough to use Samba
#		to do strange voodoo with iTunes and SlimServer.  Voodoo aside,
#		it seems reasonable enough to add this here, since it's one of
#		the only other directories that you can configure for the
#		SlimServer.
#
[ -z "${SLIM_USER}" ]			&& SLIM_USER="slimserver"
[ -z "${SLIM_GROUP}" ]			&& SLIM_GROUP="slimserver"
[ -z "${SLIM_MUSIC_DIR}" ]		&& SLIM_MUSIC_DIR="/srv/music/library"
[ -z "${SLIM_PLAYLISTS_DIR}" ]	&& SLIM_PLAYLISTS_DIR="/srv/music/playlists"
[ -z "${SLIM_ITUNES_DIR}" ]		&& SLIM_ITUNES_DIR="/srv/music/itunes"
[ -z "${SLIM_ART_DIR}" ]		&& SLIM_ART_DIR="/srv/music/artwork"
[ -z "${SLIM_INPUT_FORMATS}" ]	&& SLIM_INPUT_FORMATS="\$NONE\$"
[ -z "${SLIM_OUTPUT_FORMATS}" ]	&& SLIM_OUTPUT_FORMATS="\$NONE\$"

#
# NOTE: I disabled all references to SLIM_OTHER_GROUPS, since I believe
#		that, as a daemon, slimserver should exist in its own group by
#		default, and any other group memberships should be added by the
#		user later on, by using gpasswd.  I've added mention of that to
#		the postinst information.
#
#[ -z "${SLIM_OTHER_GROUPS}" ]	&& SLIM_OTHER_GROUPS="audio"

#
# List of variables to export to /etc/conf.d/slimserver later
#
# NOTE:	This is mostly for letting the user customize slimserver
#		the first time they install it.  I'm adding a file to
#		read these options in automagically from a file in /etc,
#		much like the apache ebuilds do for modules that ship
#		with apache.  This is my main reason for moving the config
#		to /etc/slimserver/slimserver.conf
#
SLIM_CONFIG_VARS="SLIM_INSTALL_DIR
	SLIM_USER
	SLIM_GROUP
	SLIM_MUSIC_DIR
	SLIM_PLAYLISTS_DIR
	SLIM_ITUNES_DIR
	SLIM_ART_DIR
	SLIM_INPUT_FORMATS
	SLIM_OUTPUT_FORMATS"

function configure_default_formats() {
	if [ "${SLIM_INPUT_FORMATS}" = "\$NONE\$" ]
	then
		SLIM_INPUT_FORMATS="aif wav"

		use mp3			&& SLIM_INPUT_FORMATS="${SLIM_INPUT_FORMATS} mp3"
		use vorbis		&& SLIM_INPUT_FORMATS="${SLIM_INPUT_FORMATS} ogg"
		use flac		&& SLIM_INPUT_FORMATS="${SLIM_INPUT_FORMATS} flc"
		use shorten		&& SLIM_INPUT_FORMATS="${SLIM_INPUT_FORMATS} shn"
		use musepack	&& SLIM_INPUT_FORMATS="${SLIM_INPUT_FORMATS} mpc"
		use aac			&& SLIM_INPUT_FORMATS="${SLIM_INPUT_FORMATS} mov"
		use wma			&& SLIM_INPUT_FORMATS="${SLIM_INPUT_FORMATS} wma"
	fi

	if [ "${SLIM_OUTPUT_FORMATS}" = "\$NONE\$" ]
	then
		SLIM_OUTPUT_FORMATS="aif wav"

		use encode	&& SLIM_OUTPUT_FORMATS="${SLIM_OUTPUT_FORMATS} mp3"
		use flac	&& SLIM_OUTPUT_FORMATS="${SLIM_OUTPUT_FORMATS} flc"
	fi
}

#
# This function set default settings values in init.d and conf.d files.
#
# USAGE:	replace_env_vars [src_file] [dest_file]
#
#			For each var_name in SLIM_CONFIG_VARS, this function
#			replaces instances of <<var_name>> in [src_file]
#			with the actual value of the environment variable,
#			and writes the results to [dest_file].
#
function replace_env_vars() {
	local sed_cmd="sed"
	local var_name var_value
	
	#
	# We require that SLIM_CONFIG_VARS be set, and that we are passed
	# a valid filename.
	#
	if [ ! -f "$1" -o -z "$2" -o -z "${SLIM_CONFIG_VARS}" ]
	then
		return 2
	fi
	
	# Build a sed command to do something like this:
	#    sed -e s:<SLIM_USER>:slimserver: ...
	for var_name in ${SLIM_CONFIG_VARS}
	do
		eval var_value="\${${var_name}}"
		sed_cmd="${sed_cmd} -e 's:<<${var_name}>>:${var_value}:g'"
	done

	# Apply the sed command. Output to stdout.
	cat "$1" | eval ${sed_cmd} > "$2"
}

#
# Conversion config file creator
#
# NOTE:	I completely ripped out what was here to start with, since I hope this
#		method will be more transparent to the user, produce more readable
#		convert.conf files than ship with SlimServer, and be more flexible in
#		the long run.  Here at Gentoo, we like choice, correct? =)
#
# NOTE:	The reason I'm not going to support ripping out SlimServer internal
#		support for a format is because it won't show anything if it's not
#		in convert.conf, and there's no sense in crippling internal support
#		for a format in case the user wants to add it manually later.
#
function generate_convert_conf()
{
	if [ -f "/etc/slimserver/formats" ]
	then
		cp -f "/etc/slimserver/formats" "${WORKDIR}/formats"
	else
		cp -f "${FILESDIR}/formats" "${WORKDIR}/formats"
	fi
	
	#
	# Cheating and doing it this way, since this functionality should also
	# be in userland so they don't have to deal with the nightmare that is
	# convert.conf
	#
	RUN_FROM_EBUILD=1
	CONVERT_CONF="${WORKDIR}/conversions"
	SLIM_FORMATS="${WORKDIR}/formats"
	
	source ${FILESDIR}/slimserver-formats-update
}

pkg_setup() {
	if use !encode || use !flac
	then
		ewarn "WARNING!  You are about to build media-sound/slimserver without"
		ewarn "either MP3 or FLAC support.  Most commonly a Squeezebox or"
		ewarn "SliMP3 streams in MP3, and newer Squeezebox models are capable"
		ewarn "of streaming FLAC files directly.  You probably do not want to"
		ewarn "disable these.  Here is what you have disabled, and how to"
		ewarn "fix it:"
		ewarn ""
		use !encode && ewarn "    MP3 encoding support  (USE='encode')"
		use !flac   && ewarn "    FLAC encoding support (USE='flac')"
		ewarn ""
		sleep 5
	fi

	enewgroup ${SLIM_GROUP}
	enewuser ${SLIM_USER} -1 /bin/false ${SLIM_INSTALL_DIR} ${SLIM_GROUP}
}

src_unpack() {
	unpack ${A}
	cd ${S}

	rm -rf Bin	# Shouldn't need the included static binary decoders
	rm -rf CPAN	# Shouldn't need included modules as we depend on the
				# ebuilds insteas
}

src_compile()
{
	einfo "Configuring default input and output formats"
	configure_default_formats

	einfo "Configuring boot scripts"
	replace_env_vars "${FILESDIR}/slimserver.confd"	\
		"${WORKDIR}/slimserver.confd"
	replace_env_vars "${FILESDIR}/slimserver.initd" \
		"${WORKDIR}/slimserver.initd"
	
	einfo "Generating format conversion descriptions"
	generate_convert_conf

	if [ ! -f /etc/slimserver/slimserver.conf ]
	then
		einfo "Generating slimserver.conf"
		replace_env_vars "${FILESDIR}/slimserver.conf" \
			"${WORKDIR}/slimserver.conf"
	else
		einfo "Keeping old slimserver.conf"
		cp /etc/slimserver/slimserver.conf \
			"${WORKDIR}/slimserver.conf"
	fi

	if [ ! -f /etc/slimserver/convert.conf ]
	then
		einfo "Creating new convert.conf"
		cp "${FILESDIR}/convert.conf" "${WORKDIR}/convert.conf"
	else
		einfo "Keeping old convert.conf"
		cp "/etc/slimserver/convert.conf" "${WORKDIR}/convert.conf"
	fi
}

src_install() {
	dodir "${SLIM_INSTALL_DIR}"
		cp -R "${S}"/* "${D}${SLIM_INSTALL_DIR}"
		rm -f "${D}${SLIM_INSTALL_DIR}"/convert.conf
		rm -f "${D}${SLIM_INSTALL_DIR}"/slimserver.pl
	
	dodir /etc/env.d
	insinto /etc/env.d
	doins "${FILESDIR}/99slimserver"
	
	# Note: the server script really must stay in $SLIM_INSTALL_DIR
	# (code is full of "use FindBin")
	exeinto "${SLIM_INSTALL_DIR}"
	newexe "${S}/slimserver.pl" slimserver.pl

	# Documentation:
	dodoc Installation.txt License.txt
	dohtml Changelog.html

	# Service script and its config
	dodir /etc/init.d
	newinitd "${WORKDIR}/slimserver.initd" slimserver
	dodir /etc/conf.d
	newconfd "${WORKDIR}/slimserver.confd" slimserver

	#
	# Setup the slimserver config files
	#
	dodir /etc/slimserver
	insinto /etc/slimserver
		doins	"${WORKDIR}/formats" \
				"${WORKDIR}/conversions" \
				"${WORKDIR}/slimserver.conf" \
				"${WORKDIR}/convert.conf"
		dosym /etc/slimserver/conversions "${SLIM_INSTALL_DIR}/convert.conf"
	
	keepdir "/var/log/slimserver"
	keepdir "/var/cache/slimserver"
	keepdir "/var/run/slimserver"
	
	fowners ${SLIM_USER}:${SLIM_GROUP} \
		"/var/log/slimserver" \
		"/var/cache/slimserver" \
		"/var/run/slimserver" \
		"${SLIM_INSTALL_DIR}" \
		"/etc/slimserver/slimserver.conf"
	
	chown -R ${SLIM_USER}:${SLIM_GROUP} "${D}${SLIM_INSTALL_DIR}"
	
	keepdir "${SLIM_MUSIC_DIR}"
	keepdir "${SLIM_PLAYLISTS_DIR}"
	keepdir "${SLIM_ITUNES_DIR}"
	keepdir "${SLIM_ART_DIR}"

	fowners ${SLIM_USER}:${SLIM_GROUP} \
		"${SLIM_MUSIC_DIR}" \
		"${SLIM_PLAYLISTS_DIR}" \
		"${SLIM_ITUNES_DIR}" \
		"${SLIM_ART_DIR}"
	
	fperms 0775 \
		"${SLIM_MUSIC_DIR}" \
		"${SLIM_PLAYLISTS_DIR}" \
		"${SLIM_ITUNES_DIR}" \
		"${SLIM_ART_DIR}"
	
	dosbin "${FILESDIR}/slimserver-formats-update"
}

pkg_postinst()	{
	# Very quick startup guide.
	einfo "Congratulations!  You have successfully installed the SlimServer!"
	einfo ""
	einfo "You can start and stop the server by using /etc/init.d/slimserver"
	einfo "The service can be configured by editing /etc/conf.d/slimserver"
	einfo "If you would like to start your SlimServer on every reboot, then"
	einfo "execute the following command as root:"
	einfo ""
	einfo "    rc-update add slimserver default"
	einfo ""
	einfo "The server is configured to run as ${SLIM_USER}:${SLIM_GROUP} by"
	einfo "default.  If you would like to add the user ${SLIM_USER} to other"
	einfo "system groups, do so using the following command:"
	einfo ""
	einfo "    gpasswd -a ${SLIM_USER} [group]"
	einfo ""
	einfo "For further assistance, please consult the SlimServer documentation"
	einfo "and website before reporting bugs to Gentoo."
	einfo ""
	einfo ""
	einfo ""
	einfo "If this is your first install, please run the following commands:"
	einfo ""
	einfo "    env-update"
	einfo "    source /etc/profile"
	einfo ""
}

